@@ -13,12 +13,12 @@
 # limitations under the License.
 """A geometry defined using 2 point clouds and a cost function between them."""
 import math
-from typing import Any, Callable, Optional, Tuple, Union
+from typing import Any, Callable, Literal, Optional, Tuple, Union
 
 import jax
 import jax.numpy as jnp
-from typing_extensions import Literal
 
+from ott import utils
 from ott.geometry import costs, geometry, low_rank
 from ott.math import utils as mu
 
@@ -152,7 +152,8 @@ class PointCloud(geometry.Geometry):
 
   @property
   def inv_scale_cost(self) -> float:
-    if isinstance(self._scale_cost, (int, float, jnp.DeviceArray)):
+    if isinstance(self._scale_cost,
+                  (int, float)) or utils.is_jax_array(self._scale_cost):
       return 1.0 / self._scale_cost
     self = self._masked_geom()
     if self._scale_cost == 'max_cost':
